@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,6 +43,9 @@
 class THD;
 class my_decimal;
 struct MYSQL_TIME;
+struct MYSQL_TIME_STATUS;
+
+constexpr const size_t WARN_ALLOC_BLOCK_SIZE{2048};
 
 ///////////////////////////////////////////////////////////////////////////
 
@@ -374,6 +377,11 @@ class Diagnostics_area {
     return m_message_text;
   }
 
+  uint message_text_length() const {
+    assert(m_status == DA_ERROR || m_status == DA_OK);
+    return m_message_text_length;
+  }
+
   uint mysql_errno() const {
     assert(m_status == DA_ERROR);
     return m_mysql_errno;
@@ -587,7 +595,7 @@ class Diagnostics_area {
   Diagnostics_area *m_stacked_da;
 
   /** A memory root to allocate conditions */
-  MEM_ROOT m_condition_root;
+  MEM_ROOT m_condition_root{PSI_INSTRUMENT_ME, WARN_ALLOC_BLOCK_SIZE};
 
   /** List of conditions of all severities. */
   Sql_condition_list m_conditions_list;
@@ -621,6 +629,11 @@ class Diagnostics_area {
   */
   char m_message_text[MYSQL_ERRMSG_SIZE];
 
+  /**
+    Length, in bytes, of m_message_text.
+  */
+  uint m_message_text_length;
+
   /**
     SQL RETURNED_SQLSTATE condition item.
     This member is always NUL terminated.
@@ -822,4 +835,6 @@ void warn_on_deprecated_charset(THD *thd, const CHARSET_INFO *cs,
 void warn_on_deprecated_collation(THD *thd, const CHARSET_INFO *collation,
                                   const char *option = nullptr);
 
+void check_deprecated_datetime_format(THD *thd, const CHARSET_INFO *cs,
+                                      MYSQL_TIME_STATUS &status);
 #endif  // SQL_ERROR_H
